package fr.unice.i3s.modalis.yourcast.examples;
/*******************************************************************************
 * Copyright (c) 2012 Simon Urli.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Simon Urli - initial API and implementation
 ******************************************************************************/
import java.util.*;

import fr.unice.i3s.modalis.yourcast.featureModel.FeatureModel;
import fr.unice.i3s.modalis.yourcast.multiVariant.Context;
import fr.unice.i3s.modalis.yourcast.multipleSPL.CSPL;
import fr.unice.i3s.modalis.yourcast.multipleSPL.Cardinality;
import fr.unice.i3s.modalis.yourcast.multipleSPL.FMAssociation;


public class ExampleYC {

	public static void main(String[] args) {
		
		// We define the feature models
		FeatureModel FMSource = new FeatureModel("Source", "Twitter,Picasa,Flickr");
		FeatureModel FMPolicy = new FeatureModel("Policy","Picasa-Twitter-Flickr-Truncate,sort-Picasa");
		FeatureModel FMRenderer = new FeatureModel("Renderer", "Twitter-Scrolling,Picasa-Main,Picasa-Flickr-OtherZone");
		FeatureModel FMZone = new FeatureModel("Zone", "Main-EPU-CA,Scrolling-EPU,OtherZone-CA");
		FeatureModel FMLayout = new FeatureModel("Layout", "EPU,CA");
		
		// We define the associations
		FMAssociation[] AssocSource = {new FMAssociation("Source","Renderer"), new FMAssociation("Source","Policy")};
		FMAssociation[] AssocPolicy = {new FMAssociation("Policy","Source")};
		FMAssociation[] AssocRenderer = {new FMAssociation("Renderer","Source"), new FMAssociation("Renderer","Zone")};
		FMAssociation[] AssocZone = {new FMAssociation("Zone","Renderer"), new FMAssociation("Zone","Layout")};
		FMAssociation[] AssocLayout = {new FMAssociation("Layout","Zone")};

		// We create the map between feature models' names and associations
		Map<String,List<FMAssociation>> associations = new HashMap<String,List<FMAssociation>>();
		associations.put(FMSource.getName(), Arrays.asList(AssocSource));
		associations.put(FMPolicy.getName(), Arrays.asList(AssocPolicy));
		associations.put(FMRenderer.getName(), Arrays.asList(AssocRenderer));
		associations.put(FMZone.getName(), Arrays.asList(AssocZone));
		associations.put(FMLayout.getName(), Arrays.asList(AssocLayout));
		
		// We define cardinalinities in feature models
		Map<FeatureModel, Cardinality> fmsCards = new HashMap<FeatureModel, Cardinality>();
		fmsCards.put(FMSource, Cardinality.MANY);
		fmsCards.put(FMPolicy, Cardinality.MANY);
		fmsCards.put(FMRenderer, Cardinality.MANY);
		fmsCards.put(FMZone, Cardinality.MANY);
		fmsCards.put(FMLayout, Cardinality.ONE);
		
		// We create the CSPL
		CSPL yc = new CSPL(associations, fmsCards);
		
		System.out.println("System at start : "+yc.getGlobalContext());
		
		// Make a selection consists in create a context with the selection
		
		// We select Picasa
		FeatureModel PicasaSelected = FMSource.clone();
		PicasaSelected.setFeatures("Picasa");
		Context c1 = new Context(PicasaSelected);
		yc.addContext(c1);
		
		System.out.println("The selection of picasa launch a propagation ...");
		CSPL.propagate(yc, PicasaSelected, c1);
		System.out.println("Local for Picasa : "+c1);
		System.out.println("Global for Picasa : "+yc.getGlobalContext());
		
		
		// Then we select Twitter in another context
		FeatureModel TwitterSelected = yc.getFM("Source").clone();
		TwitterSelected.setFeatures("Twitter");
		Context c2 = new Context(TwitterSelected);
		yc.addContext(c2);
		
		System.out.println("Then Twitter is selected, another propagation is launched ...");
		CSPL.propagate(yc, TwitterSelected, c2);
		System.out.println("Local for Picasa : "+c1);
		System.out.println("Local for Twitter : "+c2);
		System.out.println("Global for Picasa & Twitter : "+yc.getGlobalContext());
	}

}
