/*******************************************************************************
 * Copyright (c) 2012 Simon Urli.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Simon Urli - initial API and implementation
 ******************************************************************************/
package fr.unice.i3s.modalis.yourcast.multiVariant;
import java.util.*;

import fr.unice.i3s.modalis.yourcast.featureModel.FeatureModel;

/**
 * A context contains a map linking String and FMs. The string corresponds to the name of the feature model. 
 * Adding a feature model in a context which already have a feature model with the same name means an intersection of the two FMs will replace the old one.
 * @author urli
 *
 */
public class Context {

	private static int counter = 0;
	private String id;
	private Map<String,FeatureModel> restrictedFM;
	
	public Context(FeatureModel fm) {
		this.nameContext();
		this.restrictedFM = new HashMap<String,FeatureModel>();
		this.put(fm);
	}
	
	public Context(Map<String,FeatureModel> restrictedFM) {
		this.nameContext();
		this.restrictedFM = restrictedFM;
	}
	
	private void nameContext() {
		this.id = "Context_"+counter;
		Context.counter++;
	}
	
	public void put(FeatureModel fm) {
		FeatureModel f = fm.clone();
		if (this.restrictedFM.containsKey(fm.getName())) {
			f = this.restrictedFM.get(fm.getName());
			FeatureModel.intersectFM(f, fm);
		}
		this.restrictedFM.put(f.getName(), f);
	}
	
	public boolean containsFM(String name) {
		return this.restrictedFM.containsKey(name);
	}
	
	public FeatureModel getFM(FeatureModel fm) {
		return this.restrictedFM.get(fm.getName());
	}
	
	public FeatureModel getFMByName(String name) {
		return this.restrictedFM.get(name);
	}
	
	public List<FeatureModel> getAllFMs() {
		List<FeatureModel> res = new ArrayList<FeatureModel>();
		for (FeatureModel f : this.restrictedFM.values())
			res.add(f);
		return res;
	}
	
	public String toString() {
		String result =  this.id+" {\n";
		for (FeatureModel fm : this.restrictedFM.values())
			result += "\t"+fm+",\n";
		return result += "}";
	}
}
