/*******************************************************************************
 * Copyright (c) 2012 Simon Urli.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Public License v3.0
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Contributors:
 *     Simon Urli - initial API and implementation
 ******************************************************************************/
package fr.unice.i3s.modalis.yourcast.multipleSPL;
import fr.unice.i3s.modalis.yourcast.featureModel.FeatureModel;

/**
 * Represent an association between two feature models. Take a FM as a source, and another as a target. It also takes a restriction function.
 * For now, we consider the intersection between features is used by default.
 * @author urli
 *
 */
public class FMAssociation {

	private String source;
	private String target;
	private RestrictionFunction restrictionFunction;
	
	public FMAssociation(String s, String t) {
		this.source = s;
		this.target = t;
		this.restrictionFunction = IntersectionRestrictionFunction.instance;
	}

	/**
	 * Restrict a clone of feature model s with the feature model configuration, in using the restriction function of the association.
	 * @param s The FM target to restrict
	 * @param configuration The configuration used to compute restriction 
	 * @return the FM resulting of the restriction
	 */
	public FeatureModel restrict(FeatureModel s, FeatureModel configuration) {
		FeatureModel result = s.clone();
		this.restrictionFunction.restrictFeatureModel(result, configuration);
		return result;
	}

	public FeatureModel getTargetFM(CSPL cspl) {
		return cspl.getFM(this.target);
	}
	
	public String toString() {
		return this.source+"->"+this.target;
	}

}
